<?php
class FP_Utils {
	function get_stat_values_and_labels($floorplan_object, $args = false) {
		global $floorplans;
		$defaults = array( );
		extract( wp_parse_args( $args, $defaults ), EXTR_SKIP );

		if($exclude)
			$exclude = explode(',', $exclude);

		if($include)
			$include = explode(',', $include);
		$floorplan_stats = $floorplans['floorplan_stats'];
		foreach($floorplan_stats as $slug => $label) {
			$value = apply_filters('fp_stat_filter_' . $slug, $floorplan_object->$slug);

			//if ($slug == 'floorplan_collection') echo 'collect -'.$floorplan_object->$slug;
			// Exclude passed variables
			if(is_array($exclude) && in_array($slug, $exclude))
				continue;

			// Include only passed variables
			if(is_array($include) && in_array($slug, $include)) {
				if(!empty($value))
				$return[$label] = $value;
				continue;
			}


			if(!is_array($include)) {
			if(!empty($value))
				$return[$label] = $value;
			}

		}
		if(count($return) > 0)
			return $return;

		return false;


	}
	
	function draw_attribute_dropdown($args = '') {
		global $wp_properties, $wpdb;
		
		$defaults = array('id' => 'fp_attribute',  'name' => 'fp_attribute',  'selected' => '');
		extract( wp_parse_args( $args, $defaults ), EXTR_SKIP );

		$attributes = $floorplans['floorplan_stats'];
 		
		if(!is_array($attributes))
			return;
		
		$return = "<select id='$id' " . (!empty($name) ? " name='$name' " : '') . " >";
		foreach($attributes as $slug => $label) 
			$return .= "<option value='$slug' " . ($selected == $slug ? " selected='true' " : "") . "'>$label ($slug)</option>";
		$return .= "</select>";
	
		return $return;
	
	
	}
	
	function get_most_common_floorplan_type($array = false) {
		global $wpdb;
	
		$top_floorplan_type = $wpdb->get_row("
		SELECT meta_value as floorplan_type, count(meta_value) as count
		FROM {$wpdb->prefix}postmeta WHERE meta_key = 'floorplan_type'
		GROUP BY meta_value
		ORDER BY count DESC
		LIMIT 0,1");
		
 		
		return $top_floorplan_type->floorplan_type;
	}
	
	function settings_action($force_db = false) {
		global $floorplans, $wp_rewrite;

		// Process saving settings
		if(isset($_REQUEST['fp_settings']) && wp_verify_nonce($_REQUEST['_wpnonce'], 'fp_setting_save') ) {
 			update_option(FP_ADMIN_OPTION_NAME, $_REQUEST['fp_settings']);
			$wp_rewrite->flush_rules();

			// Load settings out of database to overwrite defaults from action_hooks.
			$floorplans_db = get_option(FP_ADMIN_OPTION_NAME);

			// Overwrite $floorplans with database setting
			$floorplans = array_merge($floorplans, $floorplans_db);

		}
		
		if($force_db) {
		
			// Load settings out of database to overwrite defaults from action_hooks.
			$floorplans_db = get_option(FP_ADMIN_OPTION_NAME);

			// Overwrite $floorplans with database setting
			$floorplans = array_merge($floorplans, $floorplans_db);

		}

		//add_filter('floorplans_image_sizes', array('FP_Utils','remove_deleted_image_sizes'));

		// Filers are applied
		/*$floorplans['configuration'] 			= apply_filters('floorplans_configuration', $floorplans['configuration']);
		$floorplans['location_matters'] 			= apply_filters('floorplans_location_matters', $floorplans['location_matters']);
		$floorplans['hidden_attributes'] 		= apply_filters('floorplans_hidden_attributes', $floorplans['hidden_attributes']);
		$floorplans['descriptions'] 				= apply_filters('floorplans_label_descriptions' , $floorplans['descriptions']);
		$floorplans['image_sizes'] 				= apply_filters('floorplans_image_sizes' , $floorplans['image_sizes']);
		$floorplans['search_conversions'] 		= apply_filters('floorplans_search_conversions' , $floorplans['search_conversions']);
		$floorplans['searchable_attributes'] 	= apply_filters('floorplans_searchable_attributes' , $floorplans['searchable_attributes']);
		$floorplans['searchable_floorplan_types'] = apply_filters('floorplans_searchable_floorplan_types' , $floorplans['searchable_floorplan_types']);
		$floorplans['floorplan_inheritance'] 		= apply_filters('FP_Utilsloorplan_inheritance' , $floorplans['floorplan_inheritance']);
		$floorplans['floorplan_meta'] 			= apply_filters('FP_Utilsloorplan_meta' , $floorplans['floorplan_meta']);
		$floorplans['floorplan_stats'] 			= apply_filters('FP_Utilsloorplan_stats' , $floorplans['floorplan_stats']);
		$floorplans['floorplan_types'] 			= apply_filters('FP_Utilsloorplan_types' , $floorplans['floorplan_types']);*/

		return $floorplans;

	}
	
	function activation() {
		global $wpdb;

		$installed_ver = get_option( "fp_version" );
		$fp_version = "0.5";

		
		// Schedule event
		update_option( "fp_version", $fp_version );
		
		// Add new attributes
		$fp_settings = get_option('fp_settings');
		
		if(empty($fp_settings['configuration']['display_address_format']))
			$fp_settings['configuration']['display_address_format'] =  "[street_number] [street_name],\n[city], [state]";

		if(empty($floorplans_settings['configuration']['google_maps_localization']))
			$fp_settings['configuration']['google_maps_localization'] =  "en";
			
		update_option('fp_settings', $fp_settings);
	}
	
	function get_floorplan($id, $args = false) {
		global $floorplans, $wpdb, $fp_cache;


 		$defaults = array("get_children" => 'true', 'return_object' => "false", 'load_gallery' => 'true', 'load_thumbnail' => 'true', 'load_parent' => 'true');
		extract( wp_parse_args( $args, $defaults ), EXTR_SKIP );
 
 
		//UD_F::log("Loading floorplan id: $id");

		$post = get_post($id, ARRAY_A);

		if($post[post_type] != 'floorplan')
			return false;



		if ( $keys = get_post_custom( $id )) {

  			foreach ( $keys as $key => $value ) {

				$keyt = trim($key);

				if ( '_' == $keyt{0} )
					continue;

				// Fix for boolean values
				switch($value[0]) {

					case 'true':
					$real_value = true;
					break;

					case 'false':
					$real_value = false;
					break;

					default:
					$real_value = $value[0];
					break;

				}
 				// if a floorplan_meta value, we do a nl2br since it will most likely have line breaks
				//if(array_key_exists($key, $floorplans['floorplan_meta']))
					//$real_value = nl2br($real_value);

				$return[$key] = $real_value;
			}
 		}


		if(is_array($return))
			$return = array_merge($return, $post);



		/*
			Figure out what the thumbnail is, and load all sizes
		*/
		if($load_thumbnail == 'true') {
			$wp_image_sizes = get_intermediate_image_sizes();

			$thumbnail_id = get_post_meta( $id, '_thumbnail_id', true );
			$attachments = get_children( array('post_parent' => $id, 'post_type' => 'attachment', 'post_mime_type' => 'image',  'orderby' => 'menu_order ASC, ID', 'order' => 'DESC') );


			if ($thumbnail_id) {
				foreach($wp_image_sizes as $image_name) {
					$this_url = wp_get_attachment_image_src( $thumbnail_id, $image_name , true );
					$return[images][$image_name] = $this_url[0];
					}

				$featured_image_id = $thumbnail_id;

			} elseif ($attachments) {
				foreach ( $attachments as $attachment_id => $attachment ) {

					foreach($wp_image_sizes as $image_name) {
						$this_url =  wp_get_attachment_image_src( $attachment_id, $image_name , true );
						$return[images][$image_name] = $this_url[0];
					}

					$featured_image_id = $attachment_id;
					break;
				}
			}


			if($featured_image_id) {
				$return[featured_image] = $featured_image_id;

				$image_title = $wpdb->get_var("SELECT post_title  FROM {$wpdb->prefix}posts WHERE ID = '$featured_image_id' ");

				$return[featured_image_title] = $image_title;
				$return[featured_image_url] = wp_get_attachment_url($featured_image_id);

			}
		} /* end load_thumbnail */


		/*
			Load all attached images and their sizes
		*/
		if($load_gallery == 'true') {
			// Get gallery images
			if($attachments) {
				foreach ( $attachments as $attachment_id => $attachment ) {
					$return[gallery][$attachment->post_name]['post_title'] = $attachment->post_title;
					foreach($wp_image_sizes as $image_name) {
						$this_url =  wp_get_attachment_image_src( $attachment_id, $image_name , true );
						$return['gallery'][$attachment->post_name][$image_name] = $this_url[0];
					}


				}

			} else {
				$return['gallery'] = false;
			}
		}
		// end load_gallery




 
		/*
			Load parent if exists.
			Inherit Parent's floorplans
		*/
		if($load_parent == 'true' && $post['post_parent']) {

			$return[is_child] = true;

			$parent_object = FP_Utils::get_floorplan($post['post_parent'], "get_children=false");


			
 			$return['parent_id'] = $post['post_parent'];
			$return['parent_link'] = $parent_object['permalink'];
			$return['parent_title'] = $parent_object['post_title'];

 
			// Inherit things
			if(is_array($floorplans['floorplan_inheritance'][$return['floorplan_type']])) {
				foreach($floorplans['floorplan_inheritance'][$return['floorplan_type']] as $inherit_attrib) {
					if(!empty($parent_object[$inherit_attrib]) && empty($return[$inherit_attrib])) {
						$return[$inherit_attrib] = $parent_object[$inherit_attrib];
					}
				}
			}
		}

		/*
			Load Children and their attributes
		*/
		if($get_children == 'true') {
		
			// Calculate variables if based off children if children exist
			$children = $wpdb->get_col("SELECT ID FROM {$wpdb->prefix}posts WHERE  post_type = 'floorplan' AND post_status = 'publish' AND post_parent = '$id' ORDER BY menu_order ASC ");

			//print_r($children);
			if(count($children) > 0) {


					// Cycle through children and get necessary variables
					foreach($children as $child_id) {

						$child_object = FP_Utils::get_floorplan($child_id, "load_parent=false");

						$return[children][$child_id] = $child_object;


						// Exclude variables from searchable attributes (to prevent ranges)
						$excluded_attributes = array(
							$floorplans['configuration']['address_attribute'],
							'city',
							'country_code',
							'country',
							'state',
							'state_code',
							'state');
							
						foreach($floorplans['searchable_attributes'] as $searchable_attribute)
 								if(!empty($child_object[$searchable_attribute]) && !in_array($searchable_attribute, $excluded_attributes))
									$range[$searchable_attribute][]	= $child_object[$searchable_attribute];


					}



				// Cycle through every type of range (i.e. price, deposit, bathroom, etc) and fix-up the respective data arrays
				foreach((array)$range as $range_attribute => $range_values) {

					// Cycle through all values of this range (attribute), and fix any ranges that use dashes
					foreach($range_values as $key => $single_value) {

						// Remove dollar signs
						$single_value = str_replace("$" , '', $single_value);

						// Fix ranges
						if(strpos($single_value, '-')) {
							$split = explode('-', $single_value);


							foreach($split as $new_single_value)

								if(!empty($new_single_value)) {
									array_push($range_values, trim($new_single_value));

								}

							// Unset original value with dash
							unset($range_values[$key]);


						}
					}

					// Remove duplicate values from this range
					$range[$range_attribute] =  array_unique($range_values);

					// Sort the values in this particular range
 					sort($range[$range_attribute]);

 					if(count($range[$range_attribute] ) < 2) {
						$return[$range_attribute] = $range[$range_attribute][0];
					}

					if(count($range[$range_attribute]) > 1) {
						$return[$range_attribute] = min($range[$range_attribute]) . " - " .  max($range[$range_attribute]);
					}

				}


			}
		} /* end get_children */



		$return['permalink'] = get_permalink($id);

		//if(empty($return['phone_number']) && !empty($floorplans['configuration']['phone_number']))
			//$return['phone_number'] = $floorplans['configuration']['phone_number'];


		if(is_array($return))
			ksort($return);

		//$return = apply_filters('floorplans_get_floorplan', $return);

		// Get rid of all empty values
		foreach($return as $key => $item) {
			if(empty($item))
				unset($return[$key]);
		}



		// Convert to object
		if($return_object == 'true') {
			$return = FP_Utils::array_to_object($return);

		}

		// Save to cache
		if(is_object($return))
			$fp_cache[$id]['object'] = $return;
		if(is_array($return))
			$fp_cache[$id]['array'] = $return;
		
		return $return;


	}
	
	function the_post($post) {
		global $post;

		if($post->post_type == 'floorplan') {
			$post = FP_Utils::get_floorplan($post->ID, "return_object=true");
		}

 	}
	
	function image_sizes_dropdown($args = "") {
		global $wp_floorplans;

		$defaults = array('name' => 'floorplans_image_sizes',  'selected' => 'none');
		extract( wp_parse_args( $args, $defaults ), EXTR_SKIP );

		if(empty($id) && !empty($name)) {
			$id = $name;
		}


		$image_array = get_intermediate_image_sizes();


		?>
			<select id="<?php echo $id ?>" name="<?php echo $name ?>" >
					<?php
						foreach($image_array as $name) {
						$sizes = FP_Utils::image_sizes($name);

						if(!$sizes)
							continue;

					?>
						<option value='<?php echo $name; ?>' <?php if($selected == $name) echo 'SELECTED'; ?>>
							 <?php echo $name; ?>: <?php echo $sizes[width]; ?>px by <?php echo $sizes[height]; ?>px
						</option>
					<?php } ?>
			</select>

		<?php
	}

	function image_sizes($type = false, $args = "") {
		global $_wp_additional_image_sizes;

		$defaults = array('return_all' => false);
		extract( wp_parse_args( $args, $defaults ), EXTR_SKIP );


		if(!$type)
			return false;

		if(is_array($_wp_additional_image_sizes[$type])) {
			$return = $_wp_additional_image_sizes[$type];

		} else {

			if($type == 'thumbnail' || $type == 'thumb') {
				$return = array('width' => intval(get_option('thumbnail_size_w')), 'height' => intval(get_option('thumbnail_size_h')));
			}

			if($type == 'medium') {
				$return = array('width' => intval(get_option('medium_size_w')), 'height' => intval(get_option('medium_size_h')));
			}

			if($type == 'large') {
				$return = array('width' => intval(get_option('large_size_w')), 'height' => intval(get_option('large_size_h')));
			}

		}

		if(!is_array($return))
			return;

		if(!$return_all) {

			// Zeroed out dimensions means they are deleted
			if(empty($return[width]) || empty($return[height]))
				return;

			// Zeroed out dimensions means they are deleted
			if($return[width] == '0' || $return[height] == '0')
				return;

		}

		// Return dimensions
		return $return;

	}
	
	function array_to_object($array = array()) {
		if (!empty($array)) {
			$data = false;
	
			foreach ($array as $akey => $aval) {
				$data -> {$akey} = $aval;
			}
	
			return $data;
		}
	
		return false;
	}
	
	function get_floorplans($args = "") {
		global $wpdb, $floorplans;

		$defaults = array('floorplan_type' => '-1');
		
		// First find out what page we are on
		if(is_array($maybe_array = unserialize($args)))
			$query = $maybe_array;
		else
			$query = wp_parse_args( $args, $defaults );
        if (substr_count($query['pagi'], '--')) {
            $pagi = explode('--', $query['pagi']);
            if(count($pagi) == 2 && is_numeric($pagi[0]) && is_numeric($pagi[1]))
                $limit_query = "LIMIT $pagi[0], $pagi[1];";
        }
		unset($query['pagi']);unset($query['pagination']);

        // sorting stuff
		if($query['sort_by']) {
            $sql_sort_by = $query['sort_by'];
            $sql_sort_order = ($query['sort_order']) ? strtoupper($query['sort_order']):'ASC';
        }
        unset($query['sort_by']);unset($query['sort_order']);
       // print_r($query);
		//echo '<br>##############################################################################<br>';
		//echo '##############################START SEARCING FOR POSTS########################<br>';
		//$matching_ids = $wpdb->get_col("SELECT post_id FROM {$wpdb->prefix}postmeta WHERE (post_id ='$matching_id_filter') AND (meta_key = 'floorplan_type' AND (meta_value ='".."'))");
		//$matching_ids = array();
		$z = 0;
        foreach ($query as $meta_key => $criteria) {
			$temp_array = array();
			
			if ($criteria == 'all')
				$temp_array = $wpdb->get_col("SELECT post_id FROM {$wpdb->prefix}postmeta WHERE (meta_key = '$meta_key')");
			else
				$temp_array = $wpdb->get_col("SELECT post_id FROM {$wpdb->prefix}postmeta WHERE (meta_key = '$meta_key' AND meta_value = '$criteria')");
			
			//print_r($temp_array);
			$arr1 = array('1', '2', '3', '4', '5');
			$arr2 = array();
			if ($z < 1)
				$matching_ids = $temp_array;
			else
				$matching_ids = array_intersect($matching_ids, $temp_array);
			//print_r($matching_ids);
			//echo '<br>-----------------------------------<br>';
			$z++;
        }
       // echo '<br>##############################################################################<br>';
		//echo '##############################DONE SEARCHING FOR POSTS########################<br>';
		$matching_ids = array_unique($matching_ids);
		//print_r($matching_ids);
		
		//for calculate total published floorplans amount 
		$total = $wpdb->get_var("SELECT COUNT(DISTINCT ID) FROM {$wpdb->prefix}posts WHERE (ID = '" . implode("' OR ID = '", $matching_ids) . "') AND post_status = 'publish'");

		// make sure selected posts are published and sorted
        if ($sql_sort_by && $sql_sort_by != 'menu_order') {
            $matching_ids = $wpdb->get_col("
                SELECT p.ID FROM {$wpdb->prefix}posts AS p, {$wpdb->prefix}postmeta AS pm
                WHERE p.ID IN (" . implode(",", $matching_ids) . ")
                    AND p.ID = pm.post_id
                    AND p.post_status = 'publish'
                    AND pm.meta_key = '$sql_sort_by'
                ORDER BY pm.meta_value + 0 $sql_sort_order
                $limit_query");
        }else{
            $matching_ids = $wpdb->get_col("
                SELECT ID FROM {$wpdb->prefix}posts
                WHERE ID IN (" . implode(",", $matching_ids) . ")
                    AND post_status = 'publish'
                ORDER BY menu_order ASC
                $limit_query");
        }

		if(!empty($matching_ids)) {
            $matching_ids['total'] = $total;
			return $matching_ids;
		}
		
		return false;
	}
	
	function is_numeric_range($array = false) {
		if(!is_array($array))
			return;
			
		foreach($array as $value) {
			if(!is_numeric($value))
				return false;	
		}
		
		return true;
	
	}
	
	function get_search_values($search_attributes, $searchable_floorplan_types, $cache = true) {
		global $wpdb, $floorplans;
		
		$cachefile = FP_Path . '/cache/searchwidget.values.res';
		if($cache && is_file($cachefile) && time() - filemtime($cachefile) < 3600) {
			$result = unserialize(file_get_contents($cachefile));
		}
		if(!$result) {
    		$query_attributes = "";
    		$query_types = "";
    		if(is_array($search_attributes))
    		    $query_attributes = "'" . implode('\',\'',$search_attributes) . "'";
    		if(is_array($searchable_floorplan_types))
    		   $query_types = "'" . implode('\',\'',$searchable_floorplan_types) . "'";
    		
    		$matching_ids = $wpdb->get_col("SELECT post_id
    			FROM {$wpdb->prefix}postmeta
    			WHERE meta_key = 'floorplan_type' AND meta_value IN ({$query_types})");
    		
    		if(empty($matching_ids))
    		    return false;
    		    
    		$matching_ids = "'" . implode('\',\'',$matching_ids) . "'";
    		$results = $wpdb->get_results("SELECT post_id, meta_key, meta_value 
    			FROM {$wpdb->prefix}postmeta
    			WHERE post_id IN ({$matching_ids}) AND meta_key IN ({$query_attributes})", ARRAY_A);
    		
    		if(empty($results))
    		    return false;
    		
    		$searchable_floorplans = array();
    		foreach ($results as $value) {
                $searchable_floorplans[$value['post_id']][$value['meta_key']] = $value['meta_value'];
    		}
    		
    		//$searchable_floorplans = floorplans_F::get_searchable_floorplans();
    		// Return fail if no searchable floorplans found
    		//if(!$searchable_floorplans)
    		//	return false;
    
     		// Cycle through all searchable floorplans all searchable data into one array
    		foreach($searchable_floorplans as $floorplan) {
    			//$floorplan = floorplans_F::get_floorplan($floorplan_id, "get_children=false&load_gallery=false");
    			foreach($floorplans['searchable_attributes'] as $searchable_attribute) {
     				// Clean up values if a conversion exists
    				$search_value = FP_Utils::do_search_conversion($searchable_attribute, trim($floorplan[$searchable_attribute]));
    				// Remove dollay signs
    				$search_value = str_replace(array(",", "$"), '', $search_value);
    				// Fix ranges
    				if(strpos($search_value, '-')) {
    					$split = explode('-', $search_value);
    					foreach($split as $new_search_value)
    						if(!empty($new_search_value))
    							$range[$searchable_attribute][] = trim($new_search_value);
    					continue;
    				}
    				
    				if(empty($search_value))
    					continue;
    				
    				$range[$searchable_attribute][]	= $search_value;
    				$range[$searchable_attribute] = array_unique($range[$searchable_attribute]);
    				sort($range[$searchable_attribute], SORT_NUMERIC);
    			}
    		}
		    $result = $range;
		    
		    $cachedir = dirname($cachefile);
            if (! is_dir($cachedir)) {
                wp_mkdir_p($cachedir);
            }
			@file_put_contents($cachefile, serialize($result));
		}
		
		return $result;
	}

    /*
        check if a search converstion exists for a attributes value
    */
	function do_search_conversion($attribute, $value, $reverse = false)  {
		global $floorplans;

		// First, check if any conversions exists for this attribute, if not, return value
		if(count($floorplans['search_conversions'][$attribute]) < 1)
			return $value;


		// If reverse is set to true, means we are trying to convert a value to integerer (most likely),
		// For isntance: in "bedrooms", $value = 0 would be converted to "Studio"
		if($reverse) {


			$flipped_conversion = array_flip($floorplans['search_conversions'][$attribute]);

			// Debug:
			//echo "reverse conversion: $attribute - $value; -" .$flipped_conversion['search_conversions'][$attribute][$value]. "<br />";


			if(!empty($flipped_conversion[$value]))
				return $flipped_conversion[$value];


		}



		// Debug:
		//echo "doing conversion: $attribute - $value; -" .$floorplans['search_conversions'][$attribute][$value]. "<br />";


		// Search conversion does exist, make sure its not an empty value.
		// Need to $conversion == '0' or else studios will not work, since they have 0 bedrooms
		$conversion = $floorplans['search_conversions'][$attribute][$value];
		if($conversion == '0' || !empty($conversion))
			return $conversion;

		// Return value in case something messed up
		return $value;


	}
}
?>